@@ -15,7 +15,7 @@ class FatalError(Exception):
 
 def parse_arguments():
     """Parse command line arguments."""
-    parser = argparse.ArgumentParser(description='Manipulate an OCFL object or inventory.',
+    parser = argparse.ArgumentParser(description='Manipulate or validate an OCFL object or inventory.',
                                      formatter_class=argparse.ArgumentDefaultsHelpFormatter)
 
     # This should really be done with add_mutually_exclusive_group() but
@@ -146,7 +146,7 @@ if __name__ == "__main__":
         args = parse_arguments()
         logging.basicConfig(level=logging.INFO if args.verbose else logging.WARN)
         do_object_operation(args)
-    except FatalError as e:
+    except (FatalError, ocfl.ObjectException) as e:
         # Show message but otherwise exit quietly
         print('Error - ' + str(e))
         sys.exit(1)
