@@ -36,6 +36,7 @@ class InventoryValidator(object):
         self.content_directory = 'content'
         self.all_versions = []
         self.manifest_files = None
+        self.unnormalized_digests = None
         self.head = 'UNKNOWN'
         # Validation control
         self.lax_digests = lax_digests
@@ -85,12 +86,12 @@ class InventoryValidator(object):
         if 'manifest' not in inventory:
             self.error("E041a")
         else:
-            self.manifest_files = self.validate_manifest(inventory['manifest'])
+            (self.manifest_files, self.unnormalized_digests) = self.validate_manifest(inventory['manifest'])
         if 'versions' not in inventory:
             self.error("E041b")
         else:
             self.all_versions = self.validate_version_sequence(inventory['versions'])
-            digests_used = self.validate_versions(inventory['versions'], self.all_versions)
+            digests_used = self.validate_versions(inventory['versions'], self.all_versions, self.unnormalized_digests)
         if 'head' in inventory:
             self.head = 'THERE_ARE_NO_VERSIONS' if len(self.all_versions) == 0 else self.all_versions[-1]
             if inventory['head'] != self.head:
@@ -99,14 +100,20 @@ class InventoryValidator(object):
             self.error("E036d")
         if 'manifest' in inventory and 'versions' in inventory:
             self.check_digests_present_and_used(self.manifest_files, digests_used)
+        if 'fixity' in inventory:
+            self.validate_fixity(inventory['fixity'], self.manifest_files)
 
     def validate_manifest(self, manifest):
         """Validate manifest block in inventory.
 
-        Returns manifest_files, a mapping from file to digest for each file in
-        the manifest.
+        Returns:
+          * manifest_files, a mapping from file to digest for each file in
+              the manifest
+          * unnormalized_digests - a set of the original digests in unnormalized
+              form that MUST match exactly the values used in state blocks
         """
         manifest_files = {}
+        unnormalized_digests = set()
         manifest_digests = set()
         if type(manifest) != dict:
             self.error('E041c')
@@ -118,17 +125,63 @@ class InventoryValidator(object):
                 elif type(manifest[digest]) != list:
                     self.error('E092', digest=digest)  # must have path list value
                 else:
+                    unnormalized_digests.add(digest)
+                    norm_digest = normalized_digest(digest, self.digest_algorithm)
+                    if norm_digest in manifest_digests:
+                        # We have already seen this in different un-normalized form!
+                        self.error("E096", digest=norm_digest)
+                    else:
+                        manifest_digests.add(norm_digest)
                     for file in manifest[digest]:
-                        norm_digest = normalized_digest(digest, self.digest_algorithm)
-                        if norm_digest in manifest_digests:
-                            # We have already seen this in different un-normalized form!
-                            self.error("E922", digest=norm_digest)
+                        manifest_files[file] = norm_digest
+                        if not self.is_valid_content_path(file):
+                            self.error("E042", path=file)
+        return (manifest_files, unnormalized_digests)
+
+    def validate_fixity(self, fixity, manifest_files):
+        """Validate fixity block in inventory.
+
+        Check the structure of the fixity block and makes sure that only files
+        listed in the manifest are referenced.
+        """
+        if type(fixity) != dict:
+            self.error('E056a')
+        else:
+            for digest_algorithm in fixity:
+                known_digest = True
+                try:
+                    regex = digest_regex(digest_algorithm)
+                except ValueError:
+                    if not self.lax_digests:
+                        self.error('E056b', algorithm=self.digest_algorithm)
+                        continue
+                    # Match anything
+                    regex = r'''^.*$'''
+                    known_digest = False
+                fixity_algoritm_block = fixity[digest_algorithm]
+                if type(fixity_algoritm_block) != dict:
+                    self.error('E057a', algorithm=self.digest_algorithm)
+                else:
+                    digests_seen = set()
+                    for digest in fixity_algoritm_block:
+                        m = re.match(regex, digest)
+                        if not m:
+                            self.error('E057b', digest=digest, algorithm=digest_algorithm)  # wrong form of digest
+                        elif type(fixity_algoritm_block[digest]) != list:
+                            self.error('E057c', digest=digest, algorithm=digest_algorithm)  # must have path list value
                         else:
-                            manifest_files[file] = norm_digest
-                            manifest_digests.add(norm_digest)
-                            if not self.is_valid_content_path(file):
-                                self.error("E042", path=file)
-        return manifest_files
+                            if known_digest:
+                                norm_digest = normalized_digest(digest, digest_algorithm)
+                            else:
+                                norm_digest = digest
+                            if norm_digest in digests_seen:
+                                # We have already seen this in different un-normalized form!
+                                self.error("E097", digest=norm_digest, algorithm=digest_algorithm)
+                            else:
+                                digests_seen.add(norm_digest)
+                            for file in fixity_algoritm_block[digest]:
+                                if file not in manifest_files:
+                                    self.error("E057d", digest=norm_digest, algorithm=digest_algorithm, path=file)
 
     def validate_version_sequence(self, versions):
         """Validate sequence of version names in versions block in inventory.
@@ -183,7 +236,7 @@ class InventoryValidator(object):
             self.error("E009")
         return all_versions
 
-    def validate_versions(self, versions, all_versions):
+    def validate_versions(self, versions, all_versions, unnormalized_digests):
         """Validate versions blocks in inventory.
 
         Requires as input two things which are assumed to be structurally correct
@@ -214,9 +267,9 @@ class InventoryValidator(object):
                 except ValueError as e:
                     self.error('E049c', version=v, description=str(e))
             if 'state' in version:
-                digests_used += self.validate_state_block(version['state'], version=v)
+                digests_used += self.validate_state_block(version['state'], version=v, unnormalized_digests=unnormalized_digests)
             else:
-                self.error('E410', version=v)
+                self.error('E048c', version=v)
             if 'message' not in version:
                 self.warn('W007a', version=v)
             elif type(version['message']) != str:
@@ -238,7 +291,7 @@ class InventoryValidator(object):
                         self.warn('W009', version=v)
         return digests_used
 
-    def validate_state_block(self, state, version):
+    def validate_state_block(self, state, version, unnormalized_digests):
         """Validate state block in a version in an inventory.
 
         The version is used only for error reporting.
@@ -261,12 +314,11 @@ class InventoryValidator(object):
                     for path in state[digest]:
                         if not self.check_logical_path(path, logical_paths, logical_directories):
                             self.error('E051', version=version, digest=digest, path=path)
+                    if digest not in unnormalized_digests:
+                        # Exact string value must match, not just normalized
+                        self.error("E050f", version=version, digest=digest)
                     norm_digest = normalized_digest(digest, self.digest_algorithm)
-                    if norm_digest in digests:
-                        # We have already seen this in different un-normalized form!
-                        self.error("E923", version=version, digest=norm_digest)
-                    else:
-                        digests.append(norm_digest)
+                    digests.append(norm_digest)
             # Check for conflicting logical paths
             for path in logical_directories:
                 if path in logical_paths:
